using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Xml.Serialization;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.Indicators;
using NinjaTrader.NinjaScript.DrawingTools;
using NinjaTrader.Cbi;
using System.Linq;  // Add this for FirstOrDefault

namespace NinjaTrader.NinjaScript.Indicators
{
    public class RRPlus : Indicator
    {
        private const string RR_TOOL_ID = "RRPlusTool";
        private const string ENTRY_TEXT_ID = "EntryText";
        private const string SL_TEXT_ID = "SLText";
        private const string TP_TEXT_ID = "TPText";
		private double lastEntryPrice;
		private double lastSlPrice;
		private int LastBarChecked = -1;

		private bool followEnabled;
		private double followStartPrice; 
		private double followSlDistance;
		private double followTpDistance;
		private int? fixedContractCount = null;
		
        private NinjaTrader.Gui.Tools.SimpleFont textFont;
        private bool objectsCreated = false;
        private Account account;

		[NinjaScriptProperty]
        [Range(0.5, 10.0)]
        [Display(Name = "Ratio", Description = "Risk-Reward ratio", Order = 1, GroupName = "RRPlus Parameters")]
        public double Ratio { get; set; }

        [NinjaScriptProperty]
        [Range(0.1, 100)]
        [Display(Name = "Risk Percentage", Description = "Maximum risk percentage per trade", Order = 2, GroupName = "RRPlus Parameters")]
        public double RiskPercentage { get; set; }

		[NinjaScriptProperty]
        [Range(0, int.MaxValue)]
        [Display(Name = "Contracts", Description = "Fixed number of contracts (overrides Risk Percentage if > 0)", Order = 3, GroupName = "RRPlus Parameters")]
        public int Contracts { get; set; }
		
		[NinjaScriptProperty]
        [Range(-1000, 1000)]
        [Display(Name = "Initial SL Distance", Description = "Initial stop loss distance in ticks (negative for buy, positive for sell)", Order = 4, GroupName = "RRPlus Parameters")]
        public int InitialSLDistance { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Show Values As Percents", Description = "Display risk/reward values as percentages instead of amounts", Order = 5, GroupName = "RRPlus Parameters")]
        public bool ShowAsPercents { get; set; }

		[NinjaScriptProperty]
		[Display(Name = "Display Points", Description = "Show distance in points", Order = 6, GroupName = "RRPlus Parameters")]
		public bool DisplayPoints { get; set; }
		
		[NinjaScriptProperty]
		[Display(Name = "Display Ticks", Description = "Show distance in ticks", Order = 7, GroupName = "RRPlus Parameters")]
		public bool DisplayTicks { get; set; }
		
		protected override void OnStateChange()
        {
            if (State == State.SetDefaults)
            {
                Description = "Risk management indicator with position sizing";
                Name = "RR Plus";
                Calculate = Calculate.OnPriceChange;
                IsOverlay = true;
                
                // Set defaults
                RiskPercentage = 0.5;
                InitialSLDistance = -150;  // Default to buy setup
                ShowAsPercents = true;
				DisplayPoints = true;
        		DisplayTicks = true;
				followEnabled = false;
				Ratio = 2.0;
				
                // Find account
                lock (Account.All)
                    account = Account.All.FirstOrDefault(a => a.Name == "Sim101");
            }
            else if (State == State.Configure)
            {
                textFont = new NinjaTrader.Gui.Tools.SimpleFont("Arial", 12);
				fixedContractCount = null;
            }
            else if (State == State.DataLoaded)
            {
                // Clean up any existing objects
                RemoveDrawObject(RR_TOOL_ID);
                RemoveDrawObject(ENTRY_TEXT_ID);
                RemoveDrawObject(SL_TEXT_ID);
                RemoveDrawObject(TP_TEXT_ID);
                objectsCreated = false;
				
				if (ChartControl != null)
		        {
		            ChartControl.MouseDown += OnChartMouseDown;
		        }
            }
			else if (State == State.Terminated)
		    {
		        if (ChartControl != null)
		        {
		            ChartControl.MouseDown -= OnChartMouseDown;
		        }
		    }
        }
		
		private bool IsUsingDefaultSlDistance(double entryPrice, double slPrice)
        {
            double defaultSlPrice = entryPrice + (InitialSLDistance * TickSize);
            // Use a small epsilon for floating point comparison
            return Math.Abs(slPrice - defaultSlPrice) < (TickSize / 2);
        }

        private double CalculateRiskBasedStopLoss(double entryPrice, int contracts)
        {
            if (account == null) return entryPrice + (InitialSLDistance * TickSize);

            double accountSize = account.Get(AccountItem.CashValue, account.Denomination);
            double maxRiskAmount = accountSize * (RiskPercentage / 100);
            
            // Calculate required distance for exact risk percentage
            double dollarPerPoint = Instrument.MasterInstrument.PointValue;
            double maxPointsRisk = maxRiskAmount / (contracts * dollarPerPoint);
            
            // Maintain the same direction as InitialSLDistance
            return InitialSLDistance < 0 ? 
                entryPrice - maxPointsRisk :  // For long positions
                entryPrice + maxPointsRisk;   // For short positions
        }

        private void OnChartMouseDown(object sender, MouseButtonEventArgs e)
        {
            try
            {
                if (State < State.Realtime || ChartControl == null || CurrentBar < 1) return;

                Point clickPoint = ChartControl.MouseDownPoint;

                // Handle entry text click for follow mode (unchanged)
                var entryTextObj = DrawObjects[ENTRY_TEXT_ID];
                if (entryTextObj != null)
                {
                    var textAnchor = entryTextObj.Anchors.First();
                    int textX = ChartControl.GetXByTime(textAnchor.Time);
                    
                    double priceRange = ChartPanel.MaxValue - ChartPanel.MinValue;
                    double pricePercent = (textAnchor.Price - ChartPanel.MinValue) / priceRange;
                    float textY = ChartPanel.Y + (ChartPanel.H * (1 - (float)pricePercent));

                    if (clickPoint.X > textX && clickPoint.X < textX + 50 && Math.Abs(clickPoint.Y - textY) < 20)
                    {
                        followEnabled = !followEnabled;
                        if (followEnabled)
                        {
                            var rrTool = DrawObjects[RR_TOOL_ID] as RiskReward;
                            if (rrTool != null)
                            {
                                double entryPrice = rrTool.EntryAnchor.Price;
                                double slPrice = rrTool.RiskAnchor.Price;
                                
                                followStartPrice = Close[0];
                                followSlDistance = slPrice - entryPrice;
                                followTpDistance = -followSlDistance * Ratio;
                            }
                        }
                        return;
                    }
                }

                // Handle SL text click
                if (!followEnabled)
		        {
		            var slTextObj = DrawObjects[SL_TEXT_ID];
		            if (slTextObj != null)
		            {
		                var slAnchor = slTextObj.Anchors.First();
		                int slX = ChartControl.GetXByTime(slAnchor.Time);
		                
		                double slPriceRange = ChartPanel.MaxValue - ChartPanel.MinValue;
		                double slPricePercent = (slAnchor.Price - ChartPanel.MinValue) / slPriceRange;
		                float slY = ChartPanel.Y + (ChartPanel.H * (1 - (float)slPricePercent));
		
		                if (clickPoint.X > slX && clickPoint.X < slX + 50 && Math.Abs(clickPoint.Y - slY) < 20)
		                {
		                    var rrTool = DrawObjects[RR_TOOL_ID] as RiskReward;
		                    if (rrTool != null)
		                    {
		                        double entryPrice = rrTool.EntryAnchor.Price;
		                        double currentSlPrice = rrTool.RiskAnchor.Price;
		                        double accountSize = account.Get(AccountItem.CashValue, account.Denomination);
		
		                        if (IsUsingDefaultSlDistance(entryPrice, currentSlPrice))
		                        {
		                            // Switch to risk-based: Lock in current contract count
		                            if (Contracts > 0)
		                            {
		                                fixedContractCount = Contracts;
		                            }
		                            else
		                            {
		                                double riskAmount = accountSize * (RiskPercentage / 100);
		                                double currentDistance = Math.Abs(currentSlPrice - entryPrice);
		                                fixedContractCount = (int)(riskAmount / (currentDistance * Instrument.MasterInstrument.PointValue));
		                            }
		
		                            // Calculate new SL for exact risk percentage
		                            double targetRiskAmount = accountSize * (RiskPercentage / 100);
		                            double dollarPerPoint = Instrument.MasterInstrument.PointValue;
		                            double requiredPointDistance = targetRiskAmount / (fixedContractCount.Value * dollarPerPoint);
		                            
		                            double newSlPrice = InitialSLDistance < 0 ? 
		                                entryPrice - requiredPointDistance :
		                                entryPrice + requiredPointDistance;
		
		                            rrTool.RiskAnchor.Price = newSlPrice;
		                        }
		                        else
		                        {
		                            // Switch back to default distance
		                            fixedContractCount = null;  // Resume dynamic calculation
		                            rrTool.RiskAnchor.Price = entryPrice + (InitialSLDistance * TickSize);
		                        }
		
		                        // Update TP
		                        double slDistance = rrTool.RiskAnchor.Price - entryPrice;
		                        double tpDistance = -slDistance * Ratio;
		                        rrTool.RewardAnchor.Price = entryPrice + tpDistance;
		                    }
		                }
		            }
		        }
            }
            catch (Exception ex)
            {
                followEnabled = false;
                Print($"Error in click handler: {ex.Message}");
                
                if (DrawObjects[RR_TOOL_ID] == null)
                {
                    objectsCreated = false;
                }
            }
        }
		

        protected override void OnBarUpdate()
        {
            if (State < State.Realtime || ChartControl == null || CurrentBar < 1) return;

            try
            {
                if (!objectsCreated && ChartBars != null)
                {
                    CreateInitialObjects();
                    objectsCreated = true;
                }
                
                if (objectsCreated)
                {
                    UpdateObjectPositions();
                }
            }
            catch (Exception ex)
            {
                Print($"Error in OnBarUpdate: {ex.Message}");
            }
        }

        private void CreateInitialObjects()
		{
		    double entryPrice = Close[0];
		    double slPrice = entryPrice + (InitialSLDistance * TickSize);
		    
		    TimeSpan barSpan = Time[0] - Time[1];
		    DateTime entryTime = Time[0].AddMinutes(22 * barSpan.TotalMinutes);
		    DateTime tpTime = Time[0].AddMinutes(2 * barSpan.TotalMinutes);
		    DateTime textTime = Time[0].AddMinutes(30 * barSpan.TotalMinutes); 
		
		    // Create RR tool
		    Draw.RiskReward(this, RR_TOOL_ID, false, entryTime, entryPrice, tpTime, slPrice, Ratio, true);
		    
		    var rrTool = DrawObjects[RR_TOOL_ID] as RiskReward;
		    if (rrTool != null)
		    {
		        rrTool.IsLocked = false;
		        // Force TP anchor position
				rrTool.RewardAnchor.Time = tpTime;
				rrTool.RiskAnchor.Time = entryTime;
		    }
		    
		    CreateTextObjects(textTime, entryPrice, slPrice);
		}

        private void CreateTextObjects(DateTime alignmentTime, double entryPrice, double slPrice)
        {
            if (account == null) return;

            int contracts;
            double accountSize = account.Get(AccountItem.CashValue, account.Denomination);
            string entryText;

            double pointsToSL = Math.Abs(entryPrice - slPrice);
            double dollarPerPoint = Instrument.MasterInstrument.PointValue;

            if (Contracts > 0)
            {
                contracts = Contracts;
            }
            else if (fixedContractCount.HasValue)
            {
                contracts = fixedContractCount.Value;
            }
            else
            {
                // Calculate contracts based on risk percentage
                double riskAmount = accountSize * (RiskPercentage / 100);
                contracts = (int)(riskAmount / (pointsToSL * dollarPerPoint));
            }

            entryText = Contracts > 0 ? 
                $"Contracts: {contracts}" :
                $"Risk: {RiskPercentage:F1}%\nContracts: {contracts}";

            // Entry text
            Draw.Text(this, ENTRY_TEXT_ID, false, entryText, alignmentTime, entryPrice, 0,
                     Brushes.Orange, textFont, TextAlignment.Left, null, null, 100);

            // Calculate risk amount based on contracts
            double actualRisk = pointsToSL * dollarPerPoint * contracts;

            // Rest of CreateTextObjects remains the same...
            string distanceText = FormatDistance(pointsToSL, pointsToSL/TickSize);
            string slText = ShowAsPercents
                ? (distanceText.Length > 0 ? distanceText + $"\nRisk: {(actualRisk/accountSize)*100:F2}%" : $"Risk: {(actualRisk/accountSize)*100:F2}%")
                : (distanceText.Length > 0 ? distanceText + $"\nRisk: {FormatCurrencyValue(actualRisk, account.Denomination)}" : $"Risk: {FormatCurrencyValue(actualRisk, account.Denomination)}");
            Draw.Text(this, SL_TEXT_ID, false, slText, alignmentTime, slPrice, 0,
                     Brushes.Red, textFont, TextAlignment.Left, null, null, 100);

            double pointsToTP = pointsToSL * Ratio;
            double tpPrice = slPrice < entryPrice ? 
                entryPrice + pointsToTP : 
                entryPrice - pointsToTP;
            
            double potentialGain = pointsToTP * dollarPerPoint * contracts;
            distanceText = FormatDistance(pointsToTP, pointsToTP/TickSize);
            string tpText = ShowAsPercents
                ? (distanceText.Length > 0 ? distanceText + $"\nGain: {(potentialGain/accountSize)*100:F2}%" : $"Gain: {(potentialGain/accountSize)*100:F2}%")
                : (distanceText.Length > 0 ? distanceText + $"\nGain: {FormatCurrencyValue(potentialGain, account.Denomination)}" : $"Gain: {FormatCurrencyValue(potentialGain, account.Denomination)}");
            Draw.Text(this, TP_TEXT_ID, false, tpText, alignmentTime, tpPrice, 0,
                     Brushes.Green, textFont, TextAlignment.Left, null, null, 100);
        }

        private void UpdateObjectPositions()
		{
		    try
		    {
		        if (CurrentBar < 1 || CurrentBars[0] < 1) return;
		        
		        TimeSpan barSpan = Time[0] - Time[1];
		        DateTime currentTime = Time[0];
		        DateTime newEntryTime = currentTime.AddMinutes(22 * barSpan.TotalMinutes);
		        DateTime newTpTime = currentTime.AddMinutes(2 * barSpan.TotalMinutes);
		        DateTime newTextTime = currentTime.AddMinutes(30 * barSpan.TotalMinutes);
		
		        var rrTool = DrawObjects[RR_TOOL_ID] as RiskReward;
		        if (rrTool != null)
		        {
		            double currentEntryPrice = rrTool.EntryAnchor.Price;
		            double currentSlPrice = rrTool.RiskAnchor.Price;
		            double currentTpPrice = rrTool.RewardAnchor.Price;
		
		            if (followEnabled)
		            {
		                rrTool.EntryAnchor.Price = Close[0];
		                currentEntryPrice = Close[0];
		                
		                rrTool.RiskAnchor.Price = Close[0] + followSlDistance;
		                currentSlPrice = rrTool.RiskAnchor.Price;
		                
		                rrTool.RewardAnchor.Price = Close[0] + followTpDistance;
		                currentTpPrice = rrTool.RewardAnchor.Price;
		            }
		
		            // Only recreate the tool on a new bar
		            if (CurrentBar != LastBarChecked)
		            {
		                LastBarChecked = CurrentBar;
		                RemoveDrawObject(RR_TOOL_ID);
		                Draw.RiskReward(this, RR_TOOL_ID, false, newEntryTime, currentEntryPrice, 
		                               newTpTime, currentSlPrice, Ratio, true);
		                
		                // Make sure the tool remains unlocked
		                var newTool = DrawObjects[RR_TOOL_ID] as RiskReward;
		                if (newTool != null)
		                {
		                    newTool.IsLocked = false;
							newTool.RiskAnchor.Time = newEntryTime;
							newTool.RewardAnchor.Time = newTpTime;
		                }
		            }
		
		            if (currentEntryPrice != lastEntryPrice || currentSlPrice != lastSlPrice)
		            {
		                lastEntryPrice = currentEntryPrice;
		                lastSlPrice = currentSlPrice;
		                CreateTextObjects(newTextTime, currentEntryPrice, currentSlPrice);
		            }
		
		            // Update text objects
		            var textObjects = new[] { ENTRY_TEXT_ID, SL_TEXT_ID, TP_TEXT_ID };
		            foreach (string id in textObjects)
		            {
		                var textObj = DrawObjects[id];
		                if (textObj != null)
		                {
		                    foreach (ChartAnchor anchor in textObj.Anchors)
		                    {
		                        anchor.Time = newTextTime;
		                        if (id == TP_TEXT_ID)
		                            anchor.Price = currentTpPrice;
		                        break;
		                    }
		                }
		            }
		        }
		    }
		    catch (Exception ex)
		    {
		        Print($"Error updating positions: {ex.Message}");
		        followEnabled = false;
		    }
		}
		
		private string FormatCurrencyValue(double value, Currency currency)
		{
		    // Add more currency symbols as needed
		    string symbol = currency == Currency.UsDollar ? "$" : "";
		    return $"{symbol}{value:F0}";
		}
		
		private string FormatDistance(double points, double ticks)
		{
		    var parts = new List<string>();
		    if (DisplayPoints)
		        parts.Add($"{points:F1} points");
		    if (DisplayTicks)
		        parts.Add($"{(int)ticks} ticks");
		    return string.Join("\n", parts);
		}
    }
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private RRPlus[] cacheRRPlus;
		public RRPlus RRPlus(double ratio, double riskPercentage, int contracts, int initialSLDistance, bool showAsPercents, bool displayPoints, bool displayTicks)
		{
			return RRPlus(Input, ratio, riskPercentage, contracts, initialSLDistance, showAsPercents, displayPoints, displayTicks);
		}

		public RRPlus RRPlus(ISeries<double> input, double ratio, double riskPercentage, int contracts, int initialSLDistance, bool showAsPercents, bool displayPoints, bool displayTicks)
		{
			if (cacheRRPlus != null)
				for (int idx = 0; idx < cacheRRPlus.Length; idx++)
					if (cacheRRPlus[idx] != null && cacheRRPlus[idx].Ratio == ratio && cacheRRPlus[idx].RiskPercentage == riskPercentage && cacheRRPlus[idx].Contracts == contracts && cacheRRPlus[idx].InitialSLDistance == initialSLDistance && cacheRRPlus[idx].ShowAsPercents == showAsPercents && cacheRRPlus[idx].DisplayPoints == displayPoints && cacheRRPlus[idx].DisplayTicks == displayTicks && cacheRRPlus[idx].EqualsInput(input))
						return cacheRRPlus[idx];
			return CacheIndicator<RRPlus>(new RRPlus(){ Ratio = ratio, RiskPercentage = riskPercentage, Contracts = contracts, InitialSLDistance = initialSLDistance, ShowAsPercents = showAsPercents, DisplayPoints = displayPoints, DisplayTicks = displayTicks }, input, ref cacheRRPlus);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.RRPlus RRPlus(double ratio, double riskPercentage, int contracts, int initialSLDistance, bool showAsPercents, bool displayPoints, bool displayTicks)
		{
			return indicator.RRPlus(Input, ratio, riskPercentage, contracts, initialSLDistance, showAsPercents, displayPoints, displayTicks);
		}

		public Indicators.RRPlus RRPlus(ISeries<double> input , double ratio, double riskPercentage, int contracts, int initialSLDistance, bool showAsPercents, bool displayPoints, bool displayTicks)
		{
			return indicator.RRPlus(input, ratio, riskPercentage, contracts, initialSLDistance, showAsPercents, displayPoints, displayTicks);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.RRPlus RRPlus(double ratio, double riskPercentage, int contracts, int initialSLDistance, bool showAsPercents, bool displayPoints, bool displayTicks)
		{
			return indicator.RRPlus(Input, ratio, riskPercentage, contracts, initialSLDistance, showAsPercents, displayPoints, displayTicks);
		}

		public Indicators.RRPlus RRPlus(ISeries<double> input , double ratio, double riskPercentage, int contracts, int initialSLDistance, bool showAsPercents, bool displayPoints, bool displayTicks)
		{
			return indicator.RRPlus(input, ratio, riskPercentage, contracts, initialSLDistance, showAsPercents, displayPoints, displayTicks);
		}
	}
}

#endregion
